library(ggplot2)
library(dplyr)
library(lixoftConnectors)

plotIndividualExposure <- function(project, outcome, cont_cov, nQuantiles = 4,
                         labels = NULL) {

  # Load a project
  initializeLixoftConnectors("simulx", force = TRUE)
  loadProject(project)

  # Get and format results
  results <- read.csv(
    file.path(tools::file_path_sans_ext(project), 
              "Endpoints",
              paste0("outcome_", outcome, ".txt"))
  )

  results$original_id <- as.character(results$original_id)
  covariates <- getGroups()[[1]]$covariate
  covariates <- getCovariateElements()[[covariates]]$data
  covariates$original_id <- covariates$ID
  covariates$ID <- NULL
  cov_list <- names(covariates)[names(covariates) != "original_id"]

  results <- left_join(results, covariates, by = "original_id")
  for (cov in cont_cov) {
    results[[cov]] <- paste(ntile(results[[cov]], nQuantiles))
  }

  summary <- NULL
  for (cov in cov_list) {
    partial_summary <- results %>% group_by_at(cov) %>%
      summarise(
        mid = quantile(.data[[outcome]], 0.5),
        lower = quantile(.data[[outcome]], 0.05),
        upper = quantile(.data[[outcome]], 0.95)
      )
    names(partial_summary)[names(partial_summary) == cov] <- "group"
    partial_summary$group <- as.character(partial_summary$group)
    partial_summary$covname <- cov
    summary <- bind_rows(summary, partial_summary)
  }

  # Calculate the ratios with respect to the median of all outcomes
  reference_value <- median(results[[outcome]])
  summary <- summary %>%
    mutate(across(mid:upper, .fns = ~.x/reference_value)) %>%
    mutate(
      LABEL = paste0(
        format(round(mid, 2), nsmall = 2),
        " [",
        format(round(lower, 2), nsmall = 2),
        "-",
        format(round(upper, 2), nsmall = 2),
        "]"
      )
    )

  summary$label <- summary$group
  if (!is.null(labels)) {
    summary$label <- apply(
      summary,
      c(1),
      function(x) return(labels[[x[["covname"]]]][[x[["label"]]]])
    )
    summary$covname <- apply(
      summary,
      c(1),
      function(x) return(labels[["covariates"]][[x[["covname"]]]])
    )
  }
  summary$label <- paste(summary$label, "\n", summary$LABEL)

  # Plot the results  
  print(
    ggplot(data = summary[summary$covname != "Reference",], aes_string(
      y = "label",
      x = "mid",
      xmin = "lower",
      xmax = "upper"
    )) +
      geom_pointrange(
        aes(color = "90% CI\nCovariate Effects"),
        size = 1,
        alpha = 1
      ) +
      annotate("rect", xmin = min(0.8), xmax = max(1.25),
               ymin = -Inf, ymax = Inf, fill = "gray", alpha=0.1) +
      geom_vline(aes(xintercept = 1, linetype = "Reference"), linetype = "dashed") + 
      facet_grid(covname ~ ., scales = "free_y", switch = "y") +
      labs(y = "", x = paste(outcome, "Relative to Reference Value"),
           colour = "", linetype = "") +
      theme_bw() +
      scale_color_manual(values = c("blue"))
  )
}

plotCovariateEffects <- function(project, outcome) {
  
  # Load a project
  initializeLixoftConnectors("simulx", force = TRUE)
  loadProject(project)
  
  # Get and format results
  results <- getEndpointsResults()$outcomes
  summary <- results[[outcome]] %>% group_by(group) %>%
    summarise(
      mid = quantile(.data[[outcome]], 0.5),
      lower = quantile(.data[[outcome]], 0.05),
      upper = quantile(.data[[outcome]], 0.95)
    )
  
  reference_value <- summary[summary$group == "Reference", ]$mid
  summary <- summary %>%
    mutate(across(mid:upper, .fns = ~.x/reference_value)) %>%
    mutate(
      LABEL = paste0(
        format(round(mid, 2), nsmall = 2),
        " [",
        format(round(lower, 2), nsmall = 2),
        "-",
        format(round(upper, 2), nsmall = 2),
        "]"
      )
    )
  
  summary$covname <- gsub("_.*", "", summary$group)
  summary$label <- gsub("^(.*?)_", "", summary$group)
  summary$label <- gsub("_", " ", summary$label) %>% paste("\n", summary$LABEL)
  
  # Plot the results  
  print(
    ggplot(data = summary[summary$covname != "Reference",], aes_string(
      y = "label",
      x = "mid",
      xmin = "lower",
      xmax = "upper"
    )) +
      geom_pointrange(
        aes(color = "90% CI\nCovariate Effects"),
        size = 1,
        alpha = 1
      ) +
      annotate("rect", xmin = min(0.8), xmax = max(1.25),
               ymin = -Inf, ymax = Inf, fill = "gray", alpha=0.1) +
      geom_vline(aes(xintercept = 1, linetype = "Reference"), linetype = "dashed") + 
      facet_grid(covname ~ ., scales = "free_y", switch = "y") +
      labs(y = "", x = paste(outcome, "Relative to Reference Value"),
           colour = "", linetype = "") +
      theme_bw() +
      scale_color_manual(values = c("blue"))
  )
}